/** title info */

#include <string.h>

#ifndef __KEYWORDS__
#define __KEYWORDS__

enum fileType_t
{
  FILE_TYPE_ERROR = -1,
  FILE_TYPE_UNINIT = 0,
  CFILE,
  CPPFILE,
  OBJCFILE,
  OBJCPPFILE,
  HDFILE,
  JFILE,
  MAX_C_FILETYPES,
  TXTFILE,
  SQLFILE,
  MKFILE,
  MAX_FILE_TYPE
};

#define CKeywords_solo "auto break case char const continue default do double "\
 "else enum extern fload for goto if int long register return short signed " \
 "sizeof static struct switch typedef union unsigned void volatile while "

/* note the words from bool to wchar_t are ANSI-C++ additions */
#define CppKeywords_solo "asm catch class delete friend inline new operator " \
 "private protected public template this throw try virtual " \
  /* ANSI-C++ additions */ \
 "bool const_cast dynamic_cast explicit export false mutable " \
 "namespace reinterpret_cast static_cast true typeid typename using wchar_t "

#define ObjCKeywords_solo "@interface @implementation @protocol @end " \
 "@private @protected @public @try @catch @finally " \
 "@class @selector @protocol @encode @synchronized " \
 "in out inout bycopy byref oneway " \
 "alloc retain release autorelease " \
 "super self "

#define CStdLibrary_solo \
  /* doesn't include: complex.h fenv.h strbool.h */ \
  /*                  stdint.h tgmath wchar or wctype.h */ \
  /* assert.h */ \
  "assert " \
  /* ctype.h */ \
  "isalnum isalpha iscntrl isdigit isgraph islower " \
  "isprint ispunct isspace isupper isxdigit tolower toupper " \
  /* errno.h */ \
  "errno " \
  /* float.h */ \
  "FLT_RADIX  FLT_MANT_DIG  DBL_MANT_DIG  LDBL_MANT_DIG  FLT_DIG " \
  "DBL_DIG LDBL_DIG  FLT_MIN_EXP  DBL_MIN_EXP  LDBL_MIN_EXP " \
  "FLT_MIN_10_EXP  DBL_MIN_10_EXP  LDBL_MIN_10_EXP " \
  "FLT_MAX_EXP  DBL_MAX_EXP  LDBL_MAX_EXP " \
  "FLT_MAX_10_EXP  DBL_MAX_10_EXP  LDBL_MAX_10_EXP " \
  "FLT_MAX DBL_MAX  LDBL_MAX  FLT_EPSILON  DBL_EPSILON " \
  "LDBL_EPSILON  FLT_MIN DBL_MIN  LDBL_MIN " \
  /* limits.h */ \
  "CHAR_BIT  SCHAR_MIN  SCHAR_MAX  UCHAR_MAX  CHAR_MIN  CHAR_MAX " \
  "MB_LEN_MAX  SHRT_MIN  SHRT_MAX  USHRT_MAX  INT_MIN  INT_MAX " \
  "UINT_MAX  LONG_MIN  LONG_MAX  ULONG_MAX " \
  /* locale.h */ \
  "setlocale localeconv lconv " \
  /* math.h */ \
  "cos sin tan acos asin atan atan2 cosh sinh tanh exp frexp ldexp " \
  "log log10 modf pow sqrt ceil fabs floor fmod " \
  /* setjmp.h */ \
  "longjmp setjmp jmp_buf " \
  /* signal.h */ \
  "signal raise sig_atomic_t " \
  /* stdarg.h */ \
  "va_list va_start va_arg va_end " \
  /* stddef.h */ \
  "ptrdiff_t size_t offsetof NULL " \
  /* stdio.h */ \
  "remove rename tmpfile tmpnam fclose fflush fopen freopen setbuf " \
  "setvbuf fprintf fscanf printf scanf sprintf sscanf vfprintf vprintf " \
  "vsprintf fgetc fgets fputc fputs getc getchar gets putc putchar puts " \
  "ungetc fread fwrite fgetpos fseek fsetpos ftell rewind clearerr " \
  "feof ferror perror EOF FILENAME_MAX TMP_MAX FILE fpos_t " \
  /* stdlib.h */ \
  "atof atoi atol strtod strtol strtoul rand srand calloc free malloc " \
  "realloc abort atexit exit getenv system bsearch qsort abs div labs ldiv " \
  "mblen mbtowc wctomb mbstowcs wcstombs EXIT_FAILURE EXIT_SUCCESS " \
  "MB_CURR_MAX RAND_MAX div_t ldiv_t " \
  /* string.h */ \
  "memcpy memmove strcpy strncpy strcat strncat memcmp strcmp strcoll " \
  "strncmp strxfrm memchr strchr strcspn strpbrk strrchr strspn strstr " \
  "strtok memset strerror strlen " \
  /* time.h */ \
  "clock difftime mktime time asctime ctime gmtime localtime strftime " \
  "CLOCLS_PER_SEC clock_t time_t tm " \
  /* others */ \
  "main " \



const char SQLMajorKeywords[] =
  "accessible add all alter analyze and as asc asensitive "
  "before between bigint binary blob both by "
  "call cascade case change char character check collate column condition connection constraint "
    "continue convert create cross current_date current_time current_timestamp current_user cursor "
  "database databases day_hour day_microsecond day_minute day_second dec decimal declare default "
    "delayed delete desc describe deterministic distinct distinctrow div double drop dual "
  "each else elseif enclosed escaped exists exit explain "
  "false fetch float float4 float8 for force foreign from fulltext "
  "goto grant group "
  "having high_priority hour_microsecond hour_minute hour_second "
  "if ignore in index infile inner inout insensitive insert int int1 int2 int3 int4 int8 integer "
    "interval into is iterate "
  "join "
  "key keys kill "
  "label leading leave left like limit linear lines load localtime localtimestamp lock long "
    "longblob longtext loop low_priority "
  "master_ssl_verify_server_cert match mediumblob mediumint mediumtext middleint minute_microsecond "
    "minute_second mod modifies "
  "natural not no_write_to_binlog null numeric "
  "on optimize option optionally or order out outer outfile "
  "precision primary procedure purge "
  "range read reads read_only read_write real references regexp release rename repeat replace "
    "require restrict return revoke right rlike "
  "schema schemas second_microsecond select sensitive separator set show smallint spatial specific "
    "sql sqlexception sqlstate sqlwarning sql_big_result sql_calc_found_rows sql_small_result ssl "
    "starting straight_join "
  "table terminated then tinyblob tinyint tinytext to trailing trigger true "
  "undo union unique unlock unsigned update upgrade usage use using utc_date utc_time utc_timestamp "
  "values varbinary varchar varcharacter varying "
  "when where while with write "
  "xor "
  "year_month "
  "zerofill";

const char SQLProcedureKeywords[] = // Not reserved words but intrinsic part of procedure definitions.
  "begin comment end";

const char SQLClientKeywords[] = // Definition of keywords only used by clients, not the server itself.
  "delimiter";

const char SQLUserKeywords[] = // Definition of own keywords, not used by MySQL.
  "edit";




/* keys are a list of C++ keywords */
const char CKeywords[] = CKeywords_solo;

const char CppKeywords[] = CKeywords_solo CppKeywords_solo;

const char ObjCKeywords[] = CKeywords_solo ObjCKeywords_solo;

const char ObjCppKeywords[] = CKeywords_solo CppKeywords_solo ObjCKeywords_solo;

/* header files are used by C/C++ and ObjC, there is no way of knowing what
 * type of header file it is, so colourize everything in .h files
 */
const char HKeywords[] = CKeywords_solo CppKeywords_solo ObjCKeywords_solo;

const char JKeywords[] = "java jar blah";

/* keys 2 are a list of the C standard Library, minus a few obscure headers */
const char CStdLibrary[] =  CStdLibrary_solo;
#endif

/*
Copyright (c) 2010, Michael Mullin <masmullin@gmail.com>
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. All advertising materials mentioning features or use of this software
   must display the following acknowledgement:
   This product includes software developed by Michael Mullin <masmullin@gmail.com>.
4. Neither the name of Michael Mullin nor the
   names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY MICHAEL MULLIN ''AS IS'' AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL MICHAEL MULLIN BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/