### title info ###
NAME=CodeAssistor

ifdef DBG
OPT_OR_DBG=-g -O0
else
OPT_OR_DBG=-O2 -w
endif

### from all dependencies, compile the target ###
COMPILE=-c $< -o $@ $(OPT_OR_DBG)

CC=gcc
ARCH=-arch i386
C_OBJC=$(CC) -x objective-c++ $(ARCH) $(gDEFs) $(INCS)
C_C=$(CC) -x c++ $(ARCH) $(gDEFs) $(INCS)
C_CPP=$(CC) -x c++ $(ARCH) $(gDEFs) $(INCS)
LD=gcc $(ARCH) -L$(BLD) -framework Cocoa
AR=ar -r

gDEFs=-DSCI_NAMESPACE -DSCI_LEXER
gDEFs+=-DMACOSX

### location of scintilla... cocoa will be in $(SCI)/cocoa ###
SCI=../scintilla

### source directories common for all platforms ###
SRC_DIRS=src src/app src/ui

### source directories solely for MACOSX ###
MAC_SRC_DIRS=src/app/mac src/ui/mac
SRC_DIRS+=$(MAC_SRC_DIRS)

### include dirs are scintilla + sources
INC_DIRS=inc inc/ui inc/app $(SCI)/include \
	$(SCI)/cocoa $(SCI)/src $(SRC_DIRS)
	
INCS=$(addprefix -I,$(INC_DIRS))

vpath %.m $(SRC_DIRS)
vpath %.mm $(SRC_DIRS)
vpath %.cpp $(SRC_DIRS)
vpath %.cxx $(SRC_DIRS)
vpath %.c $(SRC_DIRS)
vpath %.xc $(SRC_DIRS)

vpath %.h $(INC_DIRS)

BLD=build

APP=$(BLD)/$(NAME).app
TARG=$(APP)/Contents/MacOS/$(NAME)
TARG_REQS=$(BLD)/libapp.a $(BLD)/libui.a
TARG_FILES=$(TARG_REQS) $(APP)/Contents/Frameworks/Sci.framework/Sci

### targets ###

test1: $(APP) $(TARG)
	./$(TARG) tests/assign1.c

run: $(APP) $(TARG)
	open $(APP)

all: $(APP) $(TARG)

test: clean $(APP)

clean:
	-rm -rf $(BLD)

$(APP): scintilla $(BLD)
	-rm -rf $(APP)
	-mkdir $(APP)
	-mkdir $(APP)/Contents/
	-mkdir $(APP)/Contents/Frameworks/
	-mkdir $(APP)/Contents/MacOS/
	-mkdir $(APP)/Contents/Resources/
	-cp -R $(SCI)/cocoa/build/Framework/Sci.framework $(APP)/Contents/Frameworks/
	-cp -R res/ui/mac/*.nib $(APP)/Contents/Resources/
	-sed "s/EXE_NAME/$(NAME)/g" < res/ui/mac/Info.plist > $(APP)/Contents/Info.plist
	-cp res/icons/from_www.clker.com/bookcoloured.icns $(APP)/Contents/Resources/

scintilla:
ifdef DBG
	cd $(SCI)/cocoa && $(MAKE) --quiet DBG=yes -f Framework.mk all
else
	cd $(SCI)/cocoa && $(MAKE) --quiet -f Framework.mk all
endif
	@echo finished scintilla

cleansci:
	cd $(SCI)/cocoa && $(MAKE) -f Framework.mk clean

$(BLD):
	-mkdir $(BLD)

targ : $(TARG)

$(TARG) : scintilla $(APP) $(TARG_REQS)
	$(LD) -ObjC $(TARG_FILES) -o $(TARG) -lstdc++

$(BLD)/libapp.a : $(BLD)/MacAppStart.o $(BLD)/main.o
	$(AR) $@ $^

$(BLD)/libui.a : $(BLD)/MyEditor.o \
	$(BLD)/CScintillaController.o \
	$(BLD)/TXTScintillaController.o \
	$(BLD)/MKScintillaController.o \
	$(BLD)/SQLScintillaController.o \
	$(BLD)/ScintillaController.o \
	$(BLD)/SciSettersGetters.o
	$(AR) $@ $^

### build commands ###

$(BLD)/%.o : %.xc
	$(C_OBJC) $(COMPILE)

$(BLD)/%.o : %.m
	$(C_OBJC) $(COMPILE)

$(BLD)/%.o : %.mm
	$(C_OBJC) $(COMPILE)

$(BLD)/%.o : %.c
	$(C_C) $(COMPILE)

$(BLD)/%.o : %.cpp
	$(C_CPP) $(COMPILE)

$(BLD)/%.o : %.cxx
	$(C_CPP) $(COMPILE)

### dependencies for .o ###

$(BLD)/MyEditor.o : MyEditor.mm MyEditor.h keywords.h

$(BLD)/MacAppStart.o : MacAppStart.m appStart.h

$(BLD)/main.o : main.c appStart.h

$(BLD)/CScintillaController.o : CScintillaController.cpp CScintillaController.h keywords.h

$(BLD)/TXTScintillaController.o : TXTScintillaController.cpp TXTScintillaController.h keywords.h

$(BLD)/SQLScintillaController.o : SQLScintillaController.cpp SQLScintillaController.h keywords.h

$(BLD)/MKScintillaController.o : MKScintillaController.cpp MKScintillaController.h keywords.h

$(BLD)/ScintillaController.o : ScintillaController.cpp ScintillaController.h keywords.h

$(BLD)/SciSettersGetters.o : SciSettersGetters.xc SciSettersGetters.h keywords.h