#include "ScintillaController.h"
#include "SciSettersGetters.h"
#include <stdio.h>
#include <list>

using namespace std;

ScintillaController* ScintillaController::_me = NULL;

/* The CScintillaController is a singleton */
ScintillaController*
ScintillaController::getInstance()
{
  if( !_me ) _me = new ScintillaController();

  return _me;
}

void
ScintillaController::setDefaultEditor
(
  void* sci,
  fileType_t subtype
)
{
  ScintillaSend* send = ScintillaSend::getInstance();
  
    // I like Courier as my main font.  Start at a decent size
  send->set(sci,SCI_STYLESETFONT,STYLE_DEFAULT,DEFAULT_FONT);
  send->set(sci,SCI_STYLESETSIZE,STYLE_DEFAULT,INITIAL_TEXT_SIZE);
  send->set(sci,SCI_SETEDGEMODE,0,0);
  send->set(sci,SCI_STYLECLEARALL,0,0);
  send->set(sci,SCI_SETMARGINSENSITIVEN,1,1);
}

bool
ScintillaController::findText
(
  void* sci,
  const char* text,
  FindDirection dir,
  bool isRegex
) 
{
  ScintillaSend* send = ScintillaSend::getInstance();
  int rc = -1;
  int end = send->get(sci,SCI_GETLENGTH);
  int start = send->get(sci,SCI_GETCURRENTPOS);

  if( start == 0 && dir == UP ) start = end;
  else if( start == end && dir == DOWN ) start = 0;

  struct Scintilla::Sci_TextToFind ttf;
  ttf.chrg.cpMin = start;
  ttf.chrg.cpMax = dir == UP ? 0 : end;
  ttf.chrgText.cpMin = ttf.chrgText.cpMax = -1;
  ttf.lpstrText = (char*)text;

  send->set(sci,SCI_FINDTEXT,0,&ttf);
  rc = ttf.chrgText.cpMin;

  if( rc < 1 )
  {
    ttf.chrg.cpMin = dir == UP ? end : 0;
    send->set(sci,SCI_FINDTEXT,0,&ttf);
    rc = ttf.chrgText.cpMin;
  }

  if( rc > -1 )
  { 
    end = dir ? ttf.chrgText.cpMax : ttf.chrgText.cpMin;
    start =   dir ? ttf.chrgText.cpMin : ttf.chrgText.cpMax;

    int line = send->get(sci,SCI_LINEFROMPOSITION,start);
    bool isVis = true;
    do
    {
      isVis = send->get(sci,SCI_GETLINEVISIBLE, line);
      if( !isVis ) send->set(sci,SCI_TOGGLEFOLD,line,0);
    }while( !isVis );

    send->set(sci,SCI_SETSEL,start,end);
  }

  return 0;
}

bool
ScintillaController::goToLine
(
  void* sci,
  unsigned int line
)
{
  bool isVis = true;
  ScintillaSend* send = ScintillaSend::getInstance();

  do
  {
    isVis = send->get(sci,SCI_GETLINEVISIBLE, line);
    if( !isVis ) send->set(sci,SCI_TOGGLEFOLD,line,0);
  }while( !isVis );

  send->set(sci,SCI_GOTOLINE,line,0);

  return 0;
}

void
ScintillaController::foldAll(void* sci)
{
  unfoldAll(sci);

  ScintillaSend* send = ScintillaSend::getInstance();
  int maxLine = send->get(sci,SCI_GETLINECOUNT,0);

  list<int> toggles;

  for( int line = 0; line < maxLine; line++ )
  {
    int fl = send->get(sci,SCI_GETFOLDLEVEL,line);
    if( fl & SC_FOLDLEVELHEADERFLAG )
    {
      if( send->get(sci,SCI_GETLINEVISIBLE,line+1) )
      {
        toggles.push_back(line);
      }
    }
  }
  
  for( list<int>::const_iterator it = toggles.begin(); it != toggles.end(); it++ )
  {
    send->set(sci,SCI_TOGGLEFOLD,*it,0);
  }
}

void
ScintillaController::unfoldAll(void* sci)
{
  ScintillaSend* send = ScintillaSend::getInstance();

  send->set(sci,SCI_COLOURISE,0,-1);
  int maxLine = send->get(sci,SCI_GETLINECOUNT,0);

  for( int line = 0; line < maxLine; line++ )
  {
    int fl = send->get(sci,SCI_GETFOLDLEVEL,line);
    if( fl & SC_FOLDLEVELHEADERFLAG )
    {
      if( !send->get(sci,SCI_GETLINEVISIBLE,line+1) )
      {
        send->set(sci,SCI_TOGGLEFOLD,line,0);
      }
    }
  }
}
/*
Copyright (c) 2010, Michael Mullin <masmullin@gmail.com>
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. All advertising materials mentioning features or use of this software
   must display the following acknowledgement:
   This product includes software developed by Michael Mullin <masmullin@gmail.com>.
4. Neither the name of Michael Mullin nor the
   names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY MICHAEL MULLIN ''AS IS'' AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL MICHAEL MULLIN BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
