#include <Shlwapi.h>
#include <stdio.h>

#include "MyEditor.h"
#include "MyEditorHandler.h"
#include "Scintilla.h"
#include "WinFile.h"
#include "CScintillaController.h"
#include "TXTScintillaController.h"
#include "MKScintillaController.h"
#include "SQLScintillaController.h"
#include "SciSettersGetters.h"

using namespace Scintilla;

HWND g_finder = NULL;
HWND g_goto =   NULL;
HWND g_about =  NULL;

MyEditor::MyEditor(HINSTANCE app, int nCmdShow, char* file)
{
	int length = 0;
	
  _window = _editor = NULL;
	_file = NULL;
  _app = app;
	_fType = FILE_TYPE_UNINIT;
  _ttf = NULL;
  
	if( createWindow() )
  {
    if( createEditor() && createMenu() )
    {
      ShowWindow(_window, nCmdShow);
      UpdateWindow(_window);
      if( setFile(file) )
      {
        LoadTextFileToEdit(_editor,_file);
        ScintillaSend* send = ScintillaSend::getInstance();
        send->set(_editor, SCI_EMPTYUNDOBUFFER, 0, 0 );
      }else
        applyStyle(HDFILE);
				
			SetFocus(_editor);
    }
  }
}

bool
MyEditor::setFile(char* file)
{
  if( file )
  {
    char fullFile[MAX_PATH];
    int retval = GetFullPathName(file,MAX_PATH,fullFile,NULL);
    
    int length = strlen(fullFile);
    
    if( length > 0 )
    {
      char* ext = NULL;
      if( _file ) delete [] _file;
      _file = new char[length+1];
      strncpy(_file,fullFile,length);
      _file[length] = '\0';
      ext = PathFindExtension(_file);
      if( ext )
      {
        setType(ext);
        applyStyle(_fType);
      }
      memset(fullFile,'\0',MAX_PATH);
      sprintf(fullFile,"%s - CodeAssistor",PathFindFileName(_file));
      
      SetWindowText(_window,fullFile);
      
      return true;
    }
  }
  
  return false;
}

MyEditor::~MyEditor()
{
	if( _file ) delete [] _file;
}

int
MyEditor::createWindow()
{
	_window = CreateWindowEx( 0,
             g_szClassName,
             "The CodeAssistor Editor",
             WS_OVERLAPPEDWINDOW,
             CW_USEDEFAULT,
             CW_USEDEFAULT,
             720,
             700,
             NULL,
             NULL,
             _app,
             NULL );

	if( _window == NULL )
	{
		MessageBox(NULL, "Window Creation Failed!", "Error!",
			MB_ICONEXCLAMATION | MB_OK);
		return 0;
	}
  
  return 1;
}

int
MyEditor::createEditor()
{
  if( _window )
  {
    _editor = CreateWindowEx( WS_EX_CLIENTEDGE,
               "Scintilla", "", 
               WS_CHILD | WS_VISIBLE | WS_VSCROLL | WS_HSCROLL 
                | ES_MULTILINE | ES_AUTOVSCROLL | ES_AUTOHSCROLL, 
               0,
               0,
               100,
               100,
               _window,
               (HMENU)IDC_MAIN_EDIT,
               GetModuleHandle(NULL),
               NULL);
  }
  
  if(_editor == NULL)
  {
    MessageBox( _window,
                "Could not create edit box.",
                "Error",
                MB_OK | MB_ICONERROR);
    return 0;
  }

  return 1;
}

int
MyEditor::createMenu()
{
  HMENU file, edit, find, view, help;
  
  _menu = CreateMenu();
  
  file = CreatePopupMenu();
  AppendMenu(file,MF_STRING, ID_FILE_NEW, "&New...\tCtrl+N");
  AppendMenu(file,MF_STRING, ID_FILE_OPEN, "&Open...\tCtrl+O");
  AppendMenu(file,MF_STRING, ID_FILE_SAVE, "&Save...\tCtrl+S");
  AppendMenu(file,MF_STRING, ID_FILE_SAVE_AS, "S&ave As...\tCtrl+A");
  AppendMenu(file,MF_STRING, ID_FILE_CLOSE, "&Close...\tAlt+F4");
  AppendMenu(file,MF_STRING, ID_FILE_EXIT, "E&xit...\tAlt+Q");
  AppendMenu(_menu,MF_STRING|MF_POPUP, (UINT)file, "&File");
  
  edit = CreatePopupMenu();
  
  AppendMenu(edit,MF_STRING, ID_EDIT_UNDO, "Undo...\tCtrl+Z");
  AppendMenu(edit,MF_STRING, ID_EDIT_REDO, "Redo...\tCtrl+Y");
  AppendMenu(edit,MF_STRING, ID_EDIT_CUT,  "Cut...\tCtrl+X");
  AppendMenu(edit,MF_STRING, ID_EDIT_COPY, "Copy...\tCtrl+C");
  AppendMenu(edit,MF_STRING, ID_EDIT_PASTE, "Paste...\tCtrl+V");
  AppendMenu(_menu,MF_STRING|MF_POPUP,(UINT)edit,"&Edit");
  
  find = CreatePopupMenu();
    
  AppendMenu(find,MF_STRING, ID_FIND, "Find...\tCtrl+F");
  AppendMenu(find,MF_STRING, ID_FIND_PREV, "Find Previous...\tF3");
  AppendMenu(find,MF_STRING, ID_FIND_NEXT, "Find Next...\tF4");
  AppendMenu(find,MF_STRING, ID_GOTO, "Goto...\tCtrl+G");
  AppendMenu(_menu,MF_STRING|MF_POPUP,(UINT)find,"&Search");

  view = CreatePopupMenu();

  AppendMenu(view,MF_STRING, ID_VIEW_WRAP, "&Wrap...\tAlt+W");
  AppendMenu(view,MF_STRING, ID_VIEW_FOLDALL, "&Fold All...\tAlt+Z");
  AppendMenu(view,MF_STRING, ID_VIEW_UNFOLDALL, "&Unfold All...\tAlt+X");
  AppendMenu(_menu,MF_STRING|MF_POPUP,(UINT)view,"&View");

  help = CreatePopupMenu();

  AppendMenu(help,MF_STRING, ID_ABOUT, "&About");
  AppendMenu(_menu,MF_STRING|MF_POPUP,(UINT)help,"&Help");
  
  SetMenu(_window,_menu);

  return 1;
}

bool
MyEditor::isOwner(HWND hwnd)
{
  return _window == hwnd;
}

void
MyEditor::setType(char* fType)
{
  if( fType )
  {
    if( !strcmp(fType,".c") )
    {
      _fType = CFILE;
    }
    else if( !strcmp(fType,".cpp") || !strcmp(fType, ".cxx") || !strcmp(fType,".C") )
    {
      _fType = CPPFILE;
    }
    else if( !strcmp(fType,".m") )
    {
      _fType = OBJCFILE;
    }
    else if( !strcmp(fType,".mm") || !strcmp(fType,".xc") )
    {
      _fType = OBJCPPFILE;
    }
    else if( !strcmp(fType,".h") )
    {
      _fType = HDFILE;
    }
    else if( !strcmp(fType,".java") )
    {
      _fType = JFILE;
    }
    else if( !strcmp(fType,".txt") )
    {
      _fType = TXTFILE;
    }
    else if( !strcmp(fType,".mk") || !strcmp(fType,".min") || !strcmp(fType,".mak"))
    {
      _fType = MKFILE;
    }
    else if( !strcmp(fType,".sql") )
    {
      _fType = SQLFILE;
    }
    else
    {
      _fType = FILE_TYPE_ERROR;
    }
  }
}

void
MyEditor::applyStyle(fileType_t fType)
{
  ScintillaController* ed = NULL;

  switch( fType )
  {
    case CFILE:
    case CPPFILE:
    case OBJCFILE:
    case OBJCPPFILE:
    case HDFILE:
    case JFILE:
    {
      ed = CScintillaController::getInstance();
      break;
    }
    case TXTFILE:
    {
      ed = TXTScintillaController::getInstance();
      break;
    }
    case MKFILE:
    {
      ed = MKScintillaController::getInstance();
      break;
    }
    case SQLFILE:
    {
      ed = SQLScintillaController::getInstance();
      break;
    }
    default:
    ;
  }
  if( ed ) ed->setDefaultEditor(_editor,fType);
}

void
MyEditor::resizeEditor()
{
}

void
MyEditor::save(bool isNew)
{
  if( isNew || _file == NULL )
    DoFileSave(_window);
  else
    SaveTextFileFromEdit(_editor,_file);
}

void
MyEditor::find( FindDirection dir, char* ttf )
{
  if( ttf )
    setTextToFind(ttf);
    
  if( !_ttf )
    return;
  
  ScintillaController* sc = ScintillaController::getInstance();
  
  sc->findText(_editor,_ttf,dir,false);
}

void
MyEditor::setTextToFind( char* ttf )
{
  int len = strlen(ttf);
  if( !len )
    return;
    
  if( _ttf ) delete [] _ttf;
  
  _ttf = new char[len+1];
  strncpy(_ttf,ttf,len);
  _ttf[len] = '\0';
}

void
MyEditor::gotoLine( unsigned int line )
{
  ScintillaController* sc = ScintillaController::getInstance();
  
  sc->goToLine(_editor,line);
}

void
MyEditor::foldAll()
{
  ScintillaController* cont = ScintillaController::getInstance();

  cont->foldAll(_editor);
}

void
MyEditor::unfoldAll()
{
  ScintillaController* cont = ScintillaController::getInstance();

  cont->unfoldAll(_editor);
}

bool
MyEditor::isDirty( void )
{
  bool rc = false;
  ScintillaSend* send = ScintillaSend::getInstance();
  
  rc = send->get(_editor,SCI_GETMODIFY);
  
  return rc;
}

void
MyEditor::toggleWrap( void )
{

  ScintillaSend* send = ScintillaSend::getInstance();
  if( isWrap )
  {
    isWrap = false;
    send->set(_editor,SCI_SETWRAPMODE,SC_WRAP_NONE,0);
  } else {
    isWrap = true;
    send->set(_editor,SCI_SETWRAPMODE,SC_WRAP_WORD,0);
  }
}

BOOL CALLBACK AboutDialogProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam)
{
  switch(Message)
  {
    case WM_CLOSE:
      DestroyWindow(hwnd);
      EndDialog(hwnd, IDC_FIND_CANCELBUTTON);
      g_finder = NULL;
      break;
    default:
      return FALSE;
  }
  return TRUE;
}

BOOL CALLBACK FindDialogProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam)
{
    switch(Message)
    {
        case WM_INITDIALOG:
          return TRUE;
        
        case WM_CLOSE:
			    DestroyWindow(hwnd);
          break;
        case WM_COMMAND:
            switch(LOWORD(wParam))
            {
                case IDC_FIND_FINDBUTTON:
                {
                  int len = GetWindowTextLength(GetDlgItem(hwnd, IDC_FIND_TEXT));
                  if(len > 0)
                  {
                    int i;
                    char* buf;
                    MyEditor* ed;
                    MyEditorHandler* meh = MyEditorHandler::getInstance();

                    buf = (char*)GlobalAlloc(GPTR, len + 1);
                    GetDlgItemText(hwnd, IDC_FIND_TEXT, buf, len + 1);

                    ed = meh->getTopEditor();
                    
                    ed->find(DOWN,buf);
                    
                    GlobalFree((HANDLE)buf);
                   }
                  break;
                }
                case IDC_GOTO_FINDBUTTON:
                {
                  int len = GetWindowTextLength(GetDlgItem(hwnd, IDC_GOTO_TEXT));
                  if(len > 0)
                  {
                    int i;
                    int line = 0;
                    char* buf;
                    MyEditor* ed;
                    MyEditorHandler* meh = MyEditorHandler::getInstance();

                    buf = (char*)GlobalAlloc(GPTR, len + 1);
                    GetDlgItemText(hwnd, IDC_GOTO_TEXT, buf, len + 1);

                    ed = meh->getTopEditor();
                    
                    line = atoi(buf);
                    
                    ed->gotoLine(line-1);
                    ed->focus();
                    
                    GlobalFree((HANDLE)buf);
                   }
                  //break; intentionally flow
                }
                case IDC_GOTO_CANCELBUTTON:
                {
                    EndDialog(hwnd, IDC_GOTO_CANCELBUTTON);
                    g_goto = NULL;
                    break;
                }
                case IDC_FIND_CANCELBUTTON:
                {
                    EndDialog(hwnd, IDC_FIND_CANCELBUTTON);
                    g_finder = NULL;
                    break;
                }
                case ID_EDIT_PASTE:
                {
                  char* clipboard = PasteFromClipboard();
                  if( clipboard )
                  {
                    int len = strlen(clipboard);
                    SetDlgItemText(hwnd, IDC_FIND_TEXT, clipboard);
                    HWND edit = GetDlgItem(hwnd,IDC_FIND_TEXT);
                    SendMessage(edit,EM_SETSEL,len,-1);

                    delete [] clipboard;
                  }
                  break;
                }
            }
        break;
        default:
            return FALSE;
    }
    return TRUE;
}

LRESULT CALLBACK WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
  MyEditorHandler* meh = MyEditorHandler::getInstance();
  MyEditor* ed = meh->findEditor(hwnd);

  static HWND findTE = NULL;

	switch(msg)
	{
		case WM_CREATE:
		{
      break;
		}
    case WM_ACTIVATE:
    {
      switch(wParam)
      {
        case WA_ACTIVE:
        case WA_CLICKACTIVE:
          if( ed ) SetFocus(ed->getEditor());
          break;
      }
      break;
    }
		case WM_SIZE:
		{
			HWND hEdit;
			RECT rcClient;

			GetClientRect(hwnd, &rcClient);

			hEdit = GetDlgItem(hwnd, IDC_MAIN_EDIT);
			SetWindowPos(hEdit, NULL, 0, 0, rcClient.right, rcClient.bottom, SWP_NOZORDER);
		}
		break;
		case WM_CLOSE:
    {
      int id;
      SetFocus(hwnd);
      if( ed->isDirty() )
      {
        char* file = NULL;
        if( ed->getFile() )
        {
          file = new char[strlen(ed->getFile())+14];
          sprintf(file,"Save File \"%s\"?",ed->getFile());
        }else
        {
          file = new char[15];
          sprintf(file,"Save Untitled?");
        }
        
        id = MessageBox(hwnd,file,"Save",MB_YESNOCANCEL);
        if( id == IDYES )
        {
          ed->save();
          DestroyWindow(hwnd);
        } else if( id == IDNO )
        {
          DestroyWindow(hwnd);
        }
      }else
      {
        DestroyWindow(hwnd);
      }
      break;
    }
		case WM_DESTROY:
      meh->removeEditor(ed);
      if( meh->isEmpty() )
        PostQuitMessage(0);
		break;
    case WM_COMMAND:
    {
      switch(LOWORD(wParam))
      {
        case ID_FILE_NEW:
        {
          meh->createEditor();
          break;
        }
        case ID_FILE_OPEN:
        {
          DoFileOpen(hwnd);
          ScintillaSend* send = ScintillaSend::getInstance();
          send->set( ed->getEditor(), SCI_EMPTYUNDOBUFFER, 0, 0 );
          break;
        }
        case ID_FILE_SAVE:
        {
          ed->save();
          ScintillaSend* send = ScintillaSend::getInstance();
          
          send->set( ed->getEditor(), SCI_SETSAVEPOINT, 0, 0 );
          break;
        }
        case ID_FILE_SAVE_AS:
        {
          ed->save(true);
          ScintillaSend* send = ScintillaSend::getInstance();
          
          send->set( ed->getEditor(), SCI_SETSAVEPOINT, 0, 0 );
          break;
        }
        case ID_FILE_CLOSE:
        {
          PostMessage(hwnd,WM_CLOSE,0,0);
          break;
        }
        case ID_FILE_EXIT:
        {
          for( int i = 0; i < meh->editorCount(); i++ )
          {
            MyEditor* close = meh->getEditor(i);
            PostMessage(close->getWindow(),WM_CLOSE,0,0);
          }
          break;
        }
        case ID_EDIT_UNDO:
        {
          ed->undo();
          break;
        }
        case ID_EDIT_REDO:
        {
          ed->redo();
          break;
        }
        case ID_EDIT_CUT:
        {
          ed->cut();
          break;
        }
        case ID_EDIT_COPY:
        {
          ed->copy();
          break;
        }
        case ID_EDIT_PASTE:
        {
          ed->paste();
          break;
        }
        case ID_FIND:
        {
          if( g_finder == NULL )
          {
            g_finder = CreateDialog(GetModuleHandle(NULL), 
              MAKEINTRESOURCE(IDD_FIND_DIALOG), NULL, FindDialogProc);
            if( g_finder != NULL)
             ShowWindow(g_finder,SW_SHOW);
          }else
          {
            SetFocus(g_finder);
          }
          break;
        }
        case ID_FIND_NEXT:
        {
          ed->find(DOWN);
          break;
        }
        case ID_FIND_PREV:
        {
          ed->find(UP);
          break;
        }
        case ID_GOTO:
        {
          if( g_goto == NULL )
          {
            g_goto = CreateDialog(GetModuleHandle(NULL), 
              MAKEINTRESOURCE(IDD_GOTO_DIALOG), NULL, FindDialogProc);
            if( g_goto != NULL)
             ShowWindow(g_goto,SW_SHOW);
          }else
          {
            SetFocus(g_goto);
          }
          break;
        }
        case ID_VIEW_WRAP:
        {
          ed->toggleWrap();
          break;
        }
        case ID_VIEW_FOLDALL:
        {
          ed->foldAll();
          break;
        }
        case ID_VIEW_UNFOLDALL:
        {
          ed->unfoldAll();
          break;
        }
        case ID_ABOUT:
        {
          if( g_about == NULL )
          {
            g_about = CreateDialog(GetModuleHandle(NULL), 
              MAKEINTRESOURCE(IDD_ABOUT), NULL, AboutDialogProc);
            if( g_about != NULL)
             ShowWindow(g_about,SW_SHOW);
          }else
          {
            SetFocus(g_about);
          }
          break;
        }
      }
      
      switch( HIWORD(wParam) )
      {
        case EN_SETFOCUS:
        {
          //meh->setTopEditor(ed);         
          break;
        }
      }
      break;
    }
    case WM_NOTIFY:
    {
      SCNotification* scn = (SCNotification*)lParam;
      switch( scn->nmhdr.code )
      {
        case SCN_MARGINCLICK:
        {
          HWND hEdit = GetDlgItem(hwnd, IDC_MAIN_EDIT);
          int line_num = 5;
          line_num = SendMessage(hEdit,SCI_LINEFROMPOSITION,scn->position,(LPARAM)scn->position);
          SendMessage(hEdit,SCI_TOGGLEFOLD,line_num,0);
          break;
        }
        case SCN_CHARADDED:
        {
          if( scn->ch == '\n' )
          {
            HWND hEdit = GetDlgItem(hwnd, IDC_MAIN_EDIT);
            ScintillaSend* send = ScintillaSend::getInstance();

            int pos = send->get(hEdit,SCI_GETCURRENTPOS);
            int line = send->get(hEdit,SCI_LINEFROMPOSITION,pos);

            bool visAbove = send->get(hEdit,SCI_GETLINEVISIBLE,line-2);
            bool visBelow = send->get(hEdit,SCI_GETLINEVISIBLE,line+1);
            if( !visBelow ) send->set(hEdit,SCI_TOGGLEFOLD,line,0);
            if( !visAbove ) send->set(hEdit,SCI_TOGGLEFOLD,line-1,0);
          }
          break;
        }
      }
      break;
    }
    default:
      return DefWindowProc(hwnd, msg, wParam, lParam);
	}
	return 0;
}
