#include "WinFile.h"
#include "MyEditor.h"
#include "MyEditorHandler.h"

#include <stdio.h>

BOOL LoadTextFileToEdit(HWND hEdit, LPCTSTR pszFileName)
{
	HANDLE hFile;
	BOOL bSuccess = FALSE;

	hFile = CreateFile(pszFileName, GENERIC_READ, FILE_SHARE_READ, NULL,
		OPEN_EXISTING, 0, NULL);
	if(hFile != INVALID_HANDLE_VALUE)
	{
		DWORD dwFileSize;

		dwFileSize = GetFileSize(hFile, NULL);
		if(dwFileSize != 0xFFFFFFFF)
		{
			LPSTR pszFileText;

			pszFileText = (LPSTR)GlobalAlloc(GPTR, dwFileSize + 1);
			if(pszFileText != NULL)
			{
				DWORD dwRead;

				if(ReadFile(hFile, pszFileText, dwFileSize, &dwRead, NULL))
				{
					pszFileText[dwFileSize] = 0; // Add null terminator
					if(SendMessage(hEdit, SCI_SETTEXT,0,(LPARAM)pszFileText))
						bSuccess = TRUE; // It worked!
				}
				GlobalFree(pszFileText);
			}
		}
		CloseHandle(hFile);
	}
	return bSuccess;
}

void DoFileOpen(HWND hwnd)
{
	OPENFILENAME ofn;
	char szFileName[MAX_PATH] = "";

	ZeroMemory(&ofn, sizeof(ofn));

	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.hwndOwner = hwnd;
	ofn.lpstrFilter = "All Files (*.*)\0*.*\0C Files (*.c)\0*.c\0C++ Files (*.cpp,*.cxx,*.C)\0*.cpp;*.cxx;*.C\0Header Files (*.h)\0*.h\0Text Files (*.txt)\0*.txt\0";
	ofn.lpstrFile = szFileName;
	ofn.nMaxFile = MAX_PATH;
	ofn.Flags = OFN_EXPLORER | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY;
	ofn.lpstrDefExt = ".*";

	if(GetOpenFileName(&ofn))
	{
		HWND hEdit = GetDlgItem(hwnd, IDC_MAIN_EDIT);
		LoadTextFileToEdit(hEdit, szFileName);
    MyEditorHandler* meh = MyEditorHandler::getInstance();
    MyEditor* ed = meh->findEditor(hwnd);
    if( ed ) ed->setFile(szFileName);
    SetFocus(hEdit);
	}
}

BOOL SaveTextFileFromEdit(HWND hEdit, LPCTSTR pszFileName)
{
	HANDLE hFile;
	BOOL bSuccess = FALSE;

	hFile = CreateFile(pszFileName, GENERIC_WRITE, 0, NULL,
		CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
	if(hFile != INVALID_HANDLE_VALUE)
	{
		int length = 0;
    
    length = SendMessage(hEdit, SCI_GETLENGTH, 0, (LPARAM)0);
		// No need to bother if there's no text.
		if(length > 0)
		{
			LPSTR pszText;
			DWORD dwBufferSize = length + 1;

			pszText = (LPSTR)GlobalAlloc(GPTR, dwBufferSize);
			if(pszText != NULL)
			{
				SendMessage(hEdit, SCI_GETTEXT, length+1, (LPARAM)pszText );
				{
					DWORD dwWritten;

					if(WriteFile(hFile, pszText, length, &dwWritten, NULL))
						bSuccess = TRUE;
				}
				GlobalFree(pszText);
			}
		}
		CloseHandle(hFile);
	}
	return bSuccess;
}

void DoFileSave(HWND hwnd)
{
	OPENFILENAME ofn;
	char szFileName[MAX_PATH] = "";

	ZeroMemory(&ofn, sizeof(ofn));

	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.hwndOwner = hwnd;
	ofn.lpstrFilter = "Text Files (*.txt)\0*.txt\0All Files (*.*)\0*.*\0";
	ofn.lpstrFile = szFileName;
	ofn.nMaxFile = MAX_PATH;
	ofn.lpstrDefExt = "txt";
	ofn.Flags = OFN_EXPLORER | OFN_PATHMUSTEXIST | OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT;

	if(GetSaveFileName(&ofn))
	{
		HWND hEdit = GetDlgItem(hwnd, IDC_MAIN_EDIT);
		SaveTextFileFromEdit(hEdit, szFileName);
    MyEditorHandler* meh = MyEditorHandler::getInstance();
    MyEditor* ed = meh->findEditor(hwnd);
    if( ed ) ed->setFile(szFileName);
	}
}

char* PasteFromClipboard( void )
{
  LPTSTR lptstr;
  HGLOBAL hglb;
  char* rc = NULL;

  if( !IsClipboardFormatAvailable(CF_TEXT) ||
      !OpenClipboard(NULL) )
  {
    printf("no");
    return rc;
  }

  hglb = GetClipboardData(CF_TEXT);
  if( hglb != NULL )
  {
    lptstr = (LPTSTR)GlobalLock(hglb);
    if( lptstr != NULL )
    {
      rc = new char[strlen(lptstr)+1];
      strcpy(rc,lptstr);
      rc[strlen(lptstr)] = '\0';
    }
  }

  CloseClipboard();

  return rc;
}
