NAME=CodeAssistor

CC=gcc
C_C=$(CC) -x c++ $(gDEFs) $(INCS)
C_CPP=$(CC) -x c++ $(gDEFs) $(INCS)

AR=ar -r

gDEFs=-DSCI_NAMESPACE -DSCI_LEXER
gDEFs+=-DWINDOWS
SCI=../scintilla

ifdef DBG
OPT_OR_DBG=-g -O0
SUBSYS=console
else
OPT_OR_DBG=-Os -w
SUBSYS=windows
endif

### from all dependencies, compile the target ###
COMPILE=-c $< -o $@ $(OPT_OR_DBG)

BLD=build
EXE=$(NAME).exe
APP=build/$(EXE)

SRC_DIRS=src src/app src/ui
WIN_SRC_DIRS=src/app/win src/ui/win res/ui/win
SRC_DIRS+=$(WIN_SRC_DIRS)

INC_DIRS=inc inc/ui inc/app $(SCI)/include \
	$(SCI)/win32 $(SCI)/src $(SRC_DIRS)
INCS=$(addprefix -I,$(INC_DIRS))

vpath %.m $(SRC_DIRS)
vpath %.mm $(SRC_DIRS)
vpath %.cpp $(SRC_DIRS)
vpath %.cxx $(SRC_DIRS)
vpath %.c $(SRC_DIRS)
vpath %.xc $(SRC_DIRS)
vpath %.rc $(SRC_DIRS) res/icons/from_www.clker.com

vpath %.h $(INC_DIRS)
vpath %.h res/icons/from_www.clker.com


test: all
	$(APP) tests/assign1.c

install: all
	-rm -rf c:\tools\$(APP)
	-cp $(APP) c:\tools\

all: 
	$(MAKE) -f win.mk $(BLD)
	$(MAKE) -f win.mk $(APP)

clean:
	-rm -rf $(BLD)
	
cleansci:
	cd $(SCI)/win32 && $(MAKE) clean

$(APP): $(BLD) $(EXE)
	
$(BLD): scintilla
	-mkdir $(BLD)
	-cp $(SCI)/bin/*.dll $(BLD)
	
scintilla:
	-mkdir $(SCI)/bin
ifdef DBG
	cd $(SCI)/win32 && $(MAKE) --quiet DBG=yes
else
	cd $(SCI)/win32 && $(MAKE) --quiet
endif
	@echo finished scintilla	

$(EXE): $(BLD)/libapp.a $(BLD)/libui.a $(BLD)/resource.o
	g++ -o $(BLD)/$(EXE) -lmingw32 $^ -lshlwapi -lcomdlg32 -Wl,-subsystem,$(SUBSYS)
	
$(BLD)/libapp.a: $(BLD)/main.o $(BLD)/WinAppStart.o
	$(AR) $@ $^
  
$(BLD)/libui.a: $(BLD)/MyEditor.o \
	$(BLD)/MyEditorHandler.o \
	$(BLD)/CScintillaController.o \
	$(BLD)/TXTScintillaController.o \
	$(BLD)/MKScintillaController.o \
	$(BLD)/SQLScintillaController.o \
	$(BLD)/ScintillaController.o \
	$(BLD)/SciSettersGetters.o \
	$(BLD)/WinFile.o
	$(AR) $@ $^

$(BLD)/%.o : %.xc
	$(C_CPP) $(COMPILE)

$(BLD)/%.o : %.m
	$(C_OBJC) $(COMPILE)

$(BLD)/%.o : %.mm
	$(C_OBJC) $(COMPILE)

$(BLD)/%.o : %.c
	$(C_C) $(COMPILE)

$(BLD)/%.o : %.cpp
	$(C_CPP) $(COMPILE)

$(BLD)/%.o : %.cxx
	$(C_CPP) $(COMPILE)
	
$(BLD)/%.o : %.rc
	windres $< -o $@

$(BLD)/main.o:  main.c appStart.h

$(BLD)/WinAppStart.o: WinAppStart.c appStart.h

$(BLD)/MyEditor.o: MyEditor.cpp MyEditor.h resource.h

$(BLD)/MyEditorHandler.o: MyEditorHandler.cpp MyEditorHandler.h MyEditor.h

$(BLD)/WinFile.o: WinFile.cpp WinFile.h

$(BLD)/CScintillaController.o : CScintillaController.cpp CScintillaController.h keywords.h

$(BLD)/TXTScintillaController.o : TXTScintillaController.cpp TXTScintillaController.h keywords.h

$(BLD)/SQLScintillaController.o : SQLScintillaController.cpp SQLScintillaController.h keywords.h

$(BLD)/MKScintillaController.o : MKScintillaController.cpp MKScintillaController.h keywords.h

$(BLD)/ScintillaController.o : ScintillaController.cpp ScintillaController.h keywords.h

$(BLD)/SciSettersGetters.o : SciSettersGetters.xc SciSettersGetters.h keywords.h

$(BLD)/resource.o : resource.rc resource.h

$(BLD)/findres.o : findres.rc resource.h